{#
Copyright (C) 2013 Branko Majic

This file is part of Django Conntrackt documentation.

This work is licensed under the Creative Commons Attribution-ShareAlike 3.0
Unported License. To view a copy of this license, visit
http://creativecommons.org/licenses/by-sa/3.0/ or send a letter to Creative
Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
#}

{% extends "!layout.html" %}

{% block footer %}
{{ super() }}
<div class="footer">
  <a rel="license" href="http://creativecommons.org/licenses/by-sa/3.0/"><img alt="Creative Commons License" style="border-width:0; vertical-align: middle; " src="{{ pathto('_static/cc-by-sa.png', 1) }}" height="16em" /></a><br/>This work is licensed under a <a rel="license" href="http://creativecommons.org/licenses/by-sa/3.0/">Creative Commons Attribution-ShareAlike 3.0 Unported License</a>.
</div>
{% endblock %}
