{% extends "conntrackt/base.html" %}

{# For html_link #}
{% load conntrackt_tags %}

{% block content %}

<div class="row">
  <div class="span4 offset4">
    {% if form.errors %}
    <p>Wrong username or password. Please try again.</p>
    {% endif %}
    <form method="post" action="{% url "django.contrib.auth.views.login" %}">
      {% csrf_token %}
      <fieldset>
        <legend>Login</legend>
        <label class="control-label" for="id_username">{{ form.username.label_tag }} {{ form.username }}</label>
        <label class="control-label" for="id_password">{{ form.password.label_tag }} {{ form.password }}</label>
        <button type="submit" class="btn">Login</button>
        <input type="hidden" name="next" value="{{next}}" />
      </fieldset>
    </form>
  </div>
</div>
{% endblock %}

