{% extends "conntrackt/base.html" %}

{# For html_link #}
{% load conntrackt_tags %}

{% block content %}

<div class="row">
  <h1 class="span12">Welcome to Conntrackt</h1>
</div>

<div class="row">
    <div class="span12">Below you may find the list of projects that are available to you. Clicking on the project will take you to the summary page for that particular project. Clicking on an entity inside of a project will take you to the summary page of an entity.</div>
</div>

<hr />

<div class="row">
  <div class="span12">
    {% html_link "Add project" "project_create" class="btn btn-primary" %}
    {% html_link "Add location" "location_create" class="btn btn-primary" %}
  </div>
</div>

<hr />

<div class="row">

  <div class="span6">
    <h2>Projects</h2>
    <div class="well">
      {% if projects %}
        <table class="table table-striped">
          {% for project in projects %}
            <tr>
              <td style="width:99%">{% html_link project.name "project" project.id class="btn btn-link" %}</td>
              <td>{% html_link '<i class="icon-book"></i>' "project_iptables" project.id class="btn btn-link" %}</td>
              <td>{% html_link '<i class="icon-edit"></i>' "project_update" project.id class="btn btn-link" %}</td>
              <td>{% html_link '<i class="icon-remove"></i>' "project_delete" project.id class="btn btn-link" %}</td>
            </tr>
          {% endfor %}
        </table>
      {% else %}
      There are no projects defined.
      {% endif %}
    </div>     
  </div>

  <div class="span6">
    <h2>Locations</h2>
    <div class="well">
      {% if locations %}
        <table class="table table-striped">
          {% for location in locations %}
            <tr>
              <td style="width:99%">{{location.name}}</td>
              <td>{% html_link '<i class="icon-edit"></i>' "location_update" location.id class="btn btn-link" %}</td>
              <td>{% html_link '<i class="icon-remove"></i>' "location_delete" location.id class="btn btn-link" %}</td>
            </tr>
          {% endfor %}
        </table>
      {% else %}
        There are no locations defined.
      {% endif %}
    </div>
  </div>

</div>

{% endblock %}
