{% load conntrackt_tags %}
  <table class="table table-striped">
    <tr>
      <td style="width:99%"><strong>{{location.name}}</strong></td>
      <td>{% html_link '<i class="icon-book"></i>' 'project_location_iptables' project.id location.id class="btn btn-link" %}</td>
      <td></td>
      <td></td>
    </tr>
{% for entity in entities %}
    <tr>
      <td>{% html_link entity.name 'entity' entity.id class="btn btn-link" %}</td>
      <td>{% html_link '<i class="icon-list"></i>' 'entity_iptables' entity.id class="btn btn-link" %}</td>
      <td>{% html_link '<i class="icon-edit"></i>' 'entity_update' entity.id class="btn btn-link" get="next="|add:request.path %}</td>
      <td>{% html_link '<i class="icon-remove"></i>' 'entity_delete' entity.id class="btn btn-link" get="next="|add:request.path %}</td>
    </tr>
{% endfor %}
    {% with project_id=project.id|slugify location_id=location.id|slugify %}
    <tr>
      <td>{% html_link "Add entity" "entity_create" class="btn btn-primary btn-mini" get="project="|add:project_id|add:"&amp;location="|add:location_id|add:"&amp;next="|add:request.path %}</td>
      <td></td>
      <td></td>
      <td></td>
    </tr>
    {% endwith %}
  </table>

