{% extends "conntrackt/base.html" %}

{# For html_link #}
{% load conntrackt_tags %}

{% block content %}
<div class="row">
  <h1 class="span12">{{project.name}}</h1>
</div>
<hr />

{% if project.description %}
<div class="row">
  <div class="span12">
    {{project.description}}
  </div>
</div>
<hr />
{% endif %}

<div class="row">
  <div class="span12">
    {% html_link "Edit" "project_update" project.id class="btn btn-primary" %}
    {% html_link "Remove" "project_delete" project.id class="btn btn-primary" %}
    {% with project_id=project.id|slugify %}
    {% html_link "Add entity" "entity_create" class="btn btn-primary" get="project="|add:project_id|add:"&amp;next="|add:request.path %}
    {% html_link "Add communication" "communication_create" class="btn btn-primary" get="project="|add:project_id|add:"&amp;next="|add:request.path %}
    {% endwith %}
    {% html_link "Get Iptables" 'project_iptables' project.id class="btn btn-primary" %}
  </div>
</div>
<hr />
{% if location_entities %}
<div class="row">
  {% for location, entities in location_entities %}
  <div class="span4">
    <div class="well">{% include "conntrackt/location_widget.html" with location=location entities=entities %}</div>
  </div>
  {% endfor %}
</div>

{% if communications %}
<div class="row">
  <div class="span12">
    <h2>Communications</h2>
    <table class="table table-hover table-condensed table-striped">
      <thead>
        <tr><th>From</th><th>To</th><th>Protocol</th><th>Port</th></tr>
      </thead>
      <tbody>
      {% for communication in communications %}
        <tr>
          <td><span class="badge" style="background-color: {{ communication.source_color }};">O</span> {{ communication.source }}</td>
          <td><span class="badge" style="background-color: {{ communication.destination_color }};">O</span> {{ communication.destination }}</td>
          <td>{{ communication.protocol }}</td>
          <td>{{ communication.port }}</td>
        </tr>
      {% endfor %}
      </tbody>
    </table>
  </div>
</div>
{% endif %}

<div class="row">
  <div class="span12">
    <h2>Communications diagram</h2>
    <img id="project_diagram" src="{% url "project_diagram" project.id %}" alt="communications diagram" />
  </div>
</div>
{% endif %}
{% endblock %}

