{% extends "conntrackt/base.html" %}

{% block content %}
<h1>Access denied</h1>

<div class="alert alert-error">You have insufficient privileges to access this resource. Please contact your local system administrator if you believe you should have been granted access.</div>

{% endblock content %}
