{% load conntrackt_tags %}
  <table class="table table-striped">
    <tr><td><strong>{% html_link project.name 'project' project.id title=project.description %}</strong></td><td>{% html_link '<i class="icon-book"></i>' 'project_iptables' project.id class="btn btn-link pull-right" %}</td></tr>
{% for entity in project.entity_set.all %}
    <tr><td>{% html_link entity.name 'entity' entity.id class="btn btn-link" %}</td><td>{% html_link '<i class="icon-list"></i>' 'entity_iptables' entity.id class="btn btn-link pull-right" %}</td></tr>
{% endfor %}
  </table>

