/**
 * Copyright (C) 2013 Branko Majic
 *
 * This file is part of Django Conntrackt.
 *
 * Django Conntrackt is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * Django Conntrackt is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * BEGIN (search suggestions)
 */

/* Do not show search suggestions by default. */
#search-suggestions {
    display: none;
}
/* Make sure that the dropdown menu is visible by default (we hide it through
parent div instead). */
#search-suggestions .dropdown-menu {
    display: block;
    width: 100%;
}

/* Cut off long links/strings, using three periods at the end to mark it. */
#search-suggestions .dropdown-menu li a {
    text-overflow:ellipsis;
    overflow: hidden;
}

/**
 * END (search suggestions)
 */

/* Project diagram should have maximum width */
#project_diagram {
    width: 100%;
}