{% extends "conntrackt/base.html" %}

{# For html_link #}
{% load conntrackt_tags %}
{# For Bootstrapped forms #}
{% load crispy_forms_tags %}

{% block content %}
<div class="row">
  <h1 class="span12">{{headline}}</h1>
</div>
<div class="row">
  <div class="span12">
    <form action="#" method="post">
      <div class="controls controls-row">
        {% csrf_token %}
        {{ form }}
        The following entries will be removed:
        <ul>
          {{ related_items|unordered_list }}
        </ul>
        Are you sure you want to remove them?
      </div>
      <hr />
      <div class="controls">
        <button type="submit" class="btn btn-primary">Remove</button>
      </div>
    </form>
  </div>
</div>
{% endblock content %}
